/*
 * Copyright (C) 2022 Associated Universities, Inc. Washington DC, USA.
 *
 * This file is part of NRAO Workspaces.
 *
 * Workspaces is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Workspaces is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
 */

package ingest

import (
	"encoding/json"
	"fmt"
	"io/ioutil"
	"net/http"
	"path/filepath"

	"github.com/magiconair/properties"
)

type calibrationInfo struct {
	Locator string `json:"locator"`
}

/**
 * Check if an error happened and panic on it if so
 *
 * @param err An error object to report
 **/
func checkError(err error) {
	if err != nil {
		panic(err)
	}
}

/**
 * Pull a property from a CAPO property file
 *
 * @param capo_profile A string with a path to the CAPO property file
 * @param property_name A string with the name of the property to be returned
 * @return a string with the property value
 **/
func getCapoProperty(capo_profile string, property_name string) string {
	prop_file := properties.MustLoadFile(capo_profile, properties.UTF8)

	property := prop_file.GetString(property_name, "")
	if len(property) == 0 {
		panic(fmt.Sprintf("Unable to retrieve property %s from %s", property_name, capo_profile))
	}

	return property
}

/**
 * Get the locator for a VLASS image set
 *
 * @param img_set_path A string with the path to the image set
 * @param capo_profile A string with the path to the CAPO property file used to
 * get the calibration lookup URL
 * @return a string with the locator value
 **/
func GetCalibrationLocator(img_set_path string, capo_profile string) string {
	lookup_url_prop := "edu.nrao.archive.workflow.config.IngestionWorkflowSettings.calibrationLookupUrl"
	workflow_endpoint := getCapoProperty(capo_profile, lookup_url_prop)

	img_set := filepath.Base(img_set_path)

	// Make the REST call
	lookup_url := workflow_endpoint + img_set
	resp, err := http.Get(lookup_url)
	checkError(err)

	defer resp.Body.Close()

	if resp.StatusCode != http.StatusOK {
		panic(fmt.Sprintf("Error making GET request to URL %s, got status code %d", lookup_url, resp.StatusCode))
	}

	resp_bytes, err := ioutil.ReadAll(resp.Body)
	checkError(err)

	// Get the json of the response body
	var calibration calibrationInfo
	json.Unmarshal(resp_bytes, &calibration)

	return calibration.Locator
}
